import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React, { useState, useRef, useEffect } from 'react';
import PropTypes from 'prop-types';
import cx from 'classnames';
import { moleculeDropdownListSizes as SIZES } from '@s-ui/react-molecule-dropdown-list';
import { inputSizes as SELECT_SIZES } from '@s-ui/react-atom-input';
import MoleculeSelectSingleSelection from './components/SingleSelection';
import MoleculeSelectMultipleSelection from './components/MultipleSelection';
import { withOpenToggle } from '@s-ui/hoc';
import { getTarget } from '@s-ui/js/lib/react';
import { getCurrentElementFocused } from '@s-ui/js/lib/dom';
var BASE_CLASS = "sui-MoleculeSelect";
var CLASS_FOCUS = BASE_CLASS + "--focus";
var CLASS_DISABLED = "is-disabled";
var SELECT_STATES = {
  ERROR: 'error',
  SUCCESS: 'success',
  ALERT: 'alert'
};
var ENABLED_KEYS = ['Enter', 'ArrowDown', 'ArrowUp'];

var getOptionData = function getOptionData(children) {
  var optionsData = {};
  React.Children.forEach(children, function (child) {
    var _child$props = child.props,
        children = _child$props.children,
        value = _child$props.value;
    optionsData[value] = children;
  });
  return optionsData;
};

var MoleculeSelect = function MoleculeSelect(props) {
  var _cx;

  var isOpen = props.isOpen,
      onToggle = props.onToggle,
      children = props.children,
      errorState = props.errorState,
      state = props.state,
      disabled = props.disabled,
      keysSelection = props.keysSelection,
      refMoleculeSelectFromProps = props.refMoleculeSelect;
  var refMoleculeSelect = useRef(refMoleculeSelectFromProps);
  var refsMoleculeSelectOptions = useRef([]);

  var _useState = useState(getOptionData(children)),
      optionsData = _useState[0],
      setOptionsData = _useState[1];

  var _useState2 = useState(false),
      focus = _useState2[0],
      setFocus = _useState2[1];

  var extendedChildren = React.Children.toArray(children).filter(Boolean).map(function (child, index) {
    refsMoleculeSelectOptions.current[index] = React.createRef();
    return React.cloneElement(child, {
      innerRef: refsMoleculeSelectOptions.current[index],
      onSelectKey: keysSelection
    });
  });
  var className = cx(BASE_CLASS, errorState && BASE_CLASS + "--" + SELECT_STATES.ERROR, errorState === false && BASE_CLASS + "--" + SELECT_STATES.SUCCESS, state && BASE_CLASS + "--" + state, (_cx = {}, _cx[CLASS_FOCUS] = focus, _cx[CLASS_DISABLED] = disabled, _cx));

  var closeList = function closeList(ev, _ref) {
    var _ref$isOutsideEvent = _ref.isOutsideEvent,
        isOutsideEvent = _ref$isOutsideEvent === void 0 ? false : _ref$isOutsideEvent;
    onToggle(ev, {
      isOpen: false
    });

    if (!isOutsideEvent) {
      ev.preventDefault();
      ev.stopPropagation();
    }
  };

  var handleOutsideClick = function handleOutsideClick(ev) {
    if (disabled) return;

    if (refMoleculeSelect.current && !refMoleculeSelect.current.contains(ev.target)) {
      // outside click
      closeList(ev, {
        isOutsideEvent: true
      });
      setFocus(false);
    }
  };

  useEffect(function () {
    document.addEventListener('touchend', handleOutsideClick);
    document.addEventListener('mousedown', handleOutsideClick);
    return function () {
      document.removeEventListener('touchend', handleOutsideClick);
      document.removeEventListener('mousedown', handleOutsideClick);
    };
  }, []); // eslint-disable-line react-hooks/exhaustive-deps

  useEffect(function () {
    setOptionsData(getOptionData(children));
  }, [children]);

  var focusFirstOption = function focusFirstOption(ev, _ref2) {
    var options = _ref2.options;
    options[0] && options[0].focus();
    ev.preventDefault();
    ev.stopPropagation();
  };

  var handleToggle = function handleToggle(ev) {
    onToggle(ev, {});
    ev.preventDefault();
    ev.stopPropagation();
  };

  var handleKeyDown = function handleKeyDown(ev) {
    ev.persist();
    var isEnabledKey = ENABLED_KEYS.includes(ev.key);
    var options = refsMoleculeSelectOptions.current.map(getTarget);
    var domSourceEvent = ev.target;
    var domMoleculeSelect = refMoleculeSelect.current;

    if (!isOpen && isEnabledKey) {
      domSourceEvent === domMoleculeSelect && handleToggle(ev);
    } else {
      var currentElementFocused = getCurrentElementFocused();
      var isSomeOptionFocused = [].concat(options).includes(currentElementFocused);
      var key = ev.key;
      if (key === 'Escape') closeList(ev);
      if (key === 'ArrowDown' && !isSomeOptionFocused) focusFirstOption(ev, {
        options: options
      });
      var optionToFocusOn = Array.from(options).find(function (option) {
        return option && option.innerText.charAt(0).toLowerCase() === key.toLowerCase();
      });
      optionToFocusOn && optionToFocusOn.focus();
    }
  };

  var handleFocusOut = function handleFocusOut(ev) {
    setFocus(false);
  };

  var handleFocusIn = function handleFocusIn() {
    !disabled && setFocus(true);
  };

  var multiselection = props.multiselection,
      propsFromProps = _objectWithoutPropertiesLoose(props, ["multiselection"]);

  return /*#__PURE__*/React.createElement("div", {
    ref: refMoleculeSelect,
    tabIndex: "0",
    className: className,
    onKeyDown: handleKeyDown,
    onFocus: handleFocusIn,
    onBlur: handleFocusOut
  }, multiselection ? /*#__PURE__*/React.createElement(MoleculeSelectMultipleSelection, Object.assign({
    refMoleculeSelect: refMoleculeSelect,
    optionsData: optionsData
  }, propsFromProps), extendedChildren) : /*#__PURE__*/React.createElement(MoleculeSelectSingleSelection, Object.assign({
    refMoleculeSelect: refMoleculeSelect,
    optionsData: optionsData
  }, propsFromProps), extendedChildren));
};

MoleculeSelect.defaultProps = {
  disabled: false,
  keysSelection: [' ', 'Enter'],
  onChange: function onChange() {},
  onToggle: function onToggle() {},
  readOnly: false,
  selectSize: SELECT_SIZES.MEDIUM
};
export default withOpenToggle(MoleculeSelect);
export { SIZES as moleculeSelectDropdownListSizes };
export { SELECT_SIZES as moleculeSelectSizes };
export { SELECT_STATES as moleculeSelectStates };